/**
 * $Id: AbstractProclamation.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package com.inxar.affirm;

import org.inxar.affirm.*;

/**
 * Base class for Proclamation implementations.
 */
public abstract class AbstractProclamation
    implements Proclamation
{
    private static final boolean DEBUG = true;

    protected AbstractProclamation() throws Exception
    {
    }
    
    public void affirm(Affirmation affirmation)
    {
        // fetch the list for this key
        AffirmationList list = (AffirmationList)map.get(affirmation.getKey());
       	//System.out.println("list is "+list);
       	//System.out.println("key is "+affirmation.getKey());
       	// if it has not been made yet we'll make a new one
       	if (list == null) {
	    // make it
	    list = new AffirmationList(affirmation.getKey());
	    // save it
	    map.put(affirmation.getKey(), list);
       	}
       	// append this affirmation
       	list.append(affirmation);
    }

    public Data proclaim(Input input) throws ProclamationException
    {
        // we'll fill these
        DataMap data = new DataMap(map.size());
        ErrMap err = new ErrMap();

    	// get an enuerator over all the AffirmationLists
    	// in the hashtable
    	java.util.Enumeration e = map.elements();

    	while (e.hasMoreElements())
	    // validate each
            ((AffirmationList)e.nextElement()).affirm(input, data, err);

    	// check for errors
    	if (!err.isEmpty())
	    throw new ProclamationException(data, err);

    	// ok!
    	return data;
    }

    public java.util.Enumeration keys()
    {
    	return map.keys();
    }

    public java.util.Enumeration enumerate(Object key)
    {
        AffirmationList list = (AffirmationList)map.get(key);
	if (list == null)
	    return null;
    	else
	    return list.enumerator();
    }

    java.util.Hashtable map;

    private static class AffirmationList
    {
    	AffirmationList(Object key)
    	{
	    this.key = key;
    	}

    	/**
    	 * Appends the given affirmation to the end of the list
    	 */
    	void append(Affirmation affirmation)
    	{
            if (tail == null) {
            	head = new AffirmationLink(affirmation);
            	tail = head;
            } else {
            	AffirmationLink link = new AffirmationLink(affirmation);
            	tail.next = link;
            	link = tail;
            }
    	}

        /**
         * Applies all affirmations in the list.
         */
        void affirm(Input input, DataMap data, ErrMap err)
        {
	    Object val = input.get(key);
	    // grab the head of the list
	    AffirmationLink list = head;
	    // iterate each list in the list
	    while (list != null) {
                try {
		    // validate
		    val = list.affirmation.affirm(val);
		    // watch for exceptions
		} catch (AffirmationException afex) {
		    System.out.println("died on "+key);
		    // put the error message.
		    err.put(key, val, afex.getMessage());
		    // end the loop if this affirmation should cause
		    // all subsequent validation of this attribute to
		    // fail.
		    if (afex.isFatal) break;
                }
                list = list.next;
	    }
	    // add the final entry to the data...
	    data.put(key, val);
        }

    	java.util.Enumeration enumerator()
    	{
            class E1 implements java.util.Enumeration {
		E1(AffirmationLink link) {
		    this.link = link;
		}
		public boolean hasMoreElements() {
		    return link != null;
		}
		public Object nextElement() {
		    Object o = link.affirmation;
		    link = link.next;
		    return o;
		}
		AffirmationLink link;
            }
            return new E1(head);
    	}

    	Object key;
    	AffirmationLink head;
    	AffirmationLink tail;
    }

    private static class AffirmationLink
    {
    	AffirmationLink(Affirmation affirmation)
    	{
	    this.affirmation = affirmation;
    	}

    	Affirmation affirmation;
    	AffirmationLink next;
    }

    private static class DataMap
	implements Data
    {
    	DataMap(int size)
    	{
	    this.map = new java.util.Hashtable(size);
    	}

    	void put(Object key, Object val)
    	{
	    if (key != null && val != null)
		map.put(key,val);
    	}

    	public java.util.Enumeration enumerator()
    	{
	    return new DatumEnumerator(map);
    	}

    	public Datum get(Object key)
    	{
	    return new SimpleDatum(key, map.get(key));
    	}

    	public boolean getBoolean(Object key)
    	{
	    return ((Boolean)map.get(key)).booleanValue();
    	}

    	public byte getByte(Object key)
    	{
	    return ((Byte)map.get(key)).byteValue();
    	}

    	public short getShort(Object key)
    	{
	    return ((Short)map.get(key)).shortValue();
    	}

    	public int getInt(Object key)
    	{
	    return ((Integer)map.get(key)).intValue();
    	}

    	public long getLong(Object key)
    	{
	    return ((Long)map.get(key)).longValue();
    	}

    	public float getFloat(Object key)
    	{
	    return ((Float)map.get(key)).floatValue();
    	}

    	public double getDouble(Object key)
    	{
	    return ((Double)map.get(key)).doubleValue();
    	}

    	public char getChar(Object key)
    	{
	    return ((Character)map.get(key)).charValue();
    	}

	public String getString(Object key)
	{
	    return (String)map.get(key);
	}

	public Object getObject(Object key)
	{
	    return map.get(key);
	}

	public boolean isEmpty()
	{
	    return map.isEmpty();
	}

	public int size()
	{
	    return map.size();
	}

	public String toString()
	{
	    return map.toString();
	}

    	java.util.Hashtable map;
    }

    private static class ErrMap
	implements Errata
    {
    	ErrMap()
    	{
    	}

    	void put(Object key, Object val, String msg)
    	{
	    // search for it
	    ErratumLink list = find(key);

	    // did we have it?
	    if (list == null) {
		list = new ErratumLink(key, root);
		root = list;
		size++;
	    }
	    list.append(val, msg);
    	}

	public java.util.Enumeration enumerator()
	{
	    return new ErratumEnumerator(root);
	}

	public Erratum get(Object key)
	{
	    return find(key);
	}

	public ErratumLink find(Object key)
	{
	    ErratumLink list = root;
	    while (list != null) {
		if (list.key.equals(key))
		    return list;
		list = list.next;
	    }
	    return null;
	}

	public boolean isEmpty()
	{
	    return size == 0;
	}

	public int size()
	{
	    return size;
	}

	public String toString()
	{
	    //return map.toString();
	    return null;
	}

	ErratumLink root;
	int size;
    }

    private static class SimpleDatum
	implements Datum
    {
    	SimpleDatum(Object key, Object val)
    	{
	    this.key = key;
	    this.val = val;
    	}
    	public Object getKey() { return key; }
    	public Object getValue() { return val; }
    	Object key;
    	Object val;
    }

    private static class ErratumLink
	implements Erratum
    {
    	ErratumLink(Object key, ErratumLink next)
    	{
	    this.key = key;
	    this.next = next;
    	}

    	void append(Object val, String msg)
    	{
            if (head == null) {
            	head = new DetailLink(val, msg);
            	tail = head;
            } else {
            	DetailLink link = new DetailLink(val, msg);
            	tail.next =  link;
            	tail = link;
            }
	}

    	public Object getKey() { return key; }
    	public Detail details() { return head; }

    	Object key;
    	DetailLink head;
    	DetailLink tail;
    	ErratumLink next;
    }

    private static class DetailLink
	implements Detail
    {
    	DetailLink(Object val, String msg)
    	{
	    this.val = val;
	    this.msg = msg;
    	}

    	public Object getValue() { return val; }
    	public String getMessage() { return msg; }
    	public boolean hasNext() { return next != null; }
    	public Detail next() { return next; }

    	Object val;
    	String msg;
    	DetailLink next;
    }

    private static class DatumEnumerator
	implements java.util.Enumeration
    {
    	DatumEnumerator(java.util.Hashtable map)
    	{
	    this.map = map;
    	}

    	public boolean hasMoreElements()
    	{
	    return keys.hasMoreElements();
    	}

    	public Object nextElement()
    	{
	    Object key = keys.nextElement();
	    Object val = map.get(key);
	    return new SimpleDatum(key,val);
    	}

    	private java.util.Enumeration keys;
    	private java.util.Hashtable map;
    }

    private static class ErratumEnumerator
	implements java.util.Enumeration
    {
    	ErratumEnumerator(ErratumLink list)
    	{
	    this.list = list;
    	}

    	public boolean hasMoreElements()
    	{
	    return list != null;
    	}

    	public Object nextElement()
    	{
	    Erratum err = list;
	    list = list.next;
	    return err;
    	}

    	private ErratumLink list;
    }
}
