/**
 * $Id: AffirmDoubleRange.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package com.inxar.affirm;

import org.inxar.affirm.*;

/**
 * <code>Affirmation</code> implementation which checks that the value
 * associated with a certain key is a valid <code>Double</code> within
 * (inclusive) the given high and lo values.
 */
public class AffirmDoubleRange
    extends AbstractAffirmation
{
    public AffirmDoubleRange(Object key, String msg, boolean isFatal, double lo, double hi)
    {
    	super(key, msg, isFatal);
    	this.lo = lo;
    	this.hi = hi;
    }

    public AffirmDoubleRange(Object key, boolean isFatal, double lo, double hi)
    {
    	this(key, "Value must be a number between "+lo+" and "+hi, isFatal, lo, hi);
    }

    public AffirmDoubleRange(Object key, double lo, double hi)
    {
    	this(key, "Value must be a number between "+lo+" and "+hi, true, lo, hi);
    }

    public Object affirm(Object val) throws AffirmationException
    {
        double num;
        // already ok?
        if (val instanceof Double) {
            num = ((Double)val).doubleValue();
	    // if it's a string, try to parse it
        } else if (val instanceof String) {
	    try {
		num = new Double((String)val).doubleValue();
	    } catch (NumberFormatException nfex) {
		throw new AffirmationException(msg, isFatal);
	    }
       	} else {
	    throw new AffirmationException(msg, isFatal);
	}

	// now see if it;s in range
	if (num >= lo && num <= hi)
	    return new Double(num);
	else
	    throw new AffirmationException(msg, isFatal);
    }

    private double lo;
    private double hi;
}


