/**
 * $Id: AffirmLong.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package com.inxar.affirm;

import org.inxar.affirm.*;

/**
 * <code>Affirmation</code> implementation which checks that the value
 * associated with a certain key is a valid <code>Long</code>
 * object.  
 */
public class AffirmLong
    extends AbstractAffirmation
{
    public AffirmLong(Object key, String msg, boolean isFatal)
    {
    	super(key, msg, isFatal);
    }

    public AffirmLong(Object key, boolean isFatal)
    {
    	this(key, "Value must be a number between "+Long.MAX_VALUE+" and "+Long.MIN_VALUE, isFatal);
    }

    public AffirmLong(Object key)
    {
    	this(key, "Value must be a number between "+Long.MAX_VALUE+" and "+Long.MIN_VALUE, true);
    }

    public Object affirm(Object val) throws AffirmationException
    {
        // already ok?
        if (val instanceof Long)
	    return val;
        // already ok?
        else if (val instanceof Number)
	    return new Long( ((Number)val).longValue() );

        // if it's a string, try to parse it
        else if (val instanceof String) {
	    String s = (String)val;
	    try {
		val = new Long(s);
	    } catch (NumberFormatException nfex) {
		throw new AffirmationException(msg, isFatal);
	    }
       	} else {
	    throw new AffirmationException(msg, isFatal);
	}
       	return val;
    }
}


