/**
 * $Id: AffirmMatch.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package com.inxar.affirm;

import org.inxar.affirm.*;

/**
 * <code>Affirmation</code> implementation which checks that the value
 * associated with a certain key is a valid <code>String</code> which
 * matches the given regular expression.  The regular expression test
 * is encapsulated by <code>gnu.regexp.RE.isMatch(Object
 * value)</code>.
 */
public class AffirmMatch
    extends AbstractAffirmation
{
    public AffirmMatch(Object key, String msg, boolean isFatal, String pattern) throws Exception
    {
    	super(key, msg, isFatal);
    	this.regex = new gnu.regexp.RE(pattern);
    }

    public AffirmMatch(Object key, boolean isFatal, String pattern) throws Exception
    {
    	this(key, "Value did not match the required pattern", isFatal, pattern);
    }

    public AffirmMatch(Object key, String pattern) throws Exception
    {
    	this(key, "Value did not match the required pattern", true, pattern);
    }

    public Object affirm(Object val) throws AffirmationException
    {
        // already ok?
        if (val instanceof String) {
	    if (regex.isMatch(val))
		return val;
	    else
		throw new AffirmationException(msg, isFatal);
       	} else {
	    throw new AffirmationException(msg, isFatal);
	}
    }

    private gnu.regexp.RE regex;
}


