/**
 * $Id: AffirmStringLength.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package com.inxar.affirm;

import org.inxar.affirm.*;

/**
 * <code>Affirmation</code> implementation which checks that the value
 * associated with a certain key is a valid <code>String</code> object
 * having legnth inclusive to the give min and max values.
 */
public class AffirmStringLength
    extends AbstractAffirmation
{
    public AffirmStringLength(Object key, String msg, boolean isFatal, int min, int max)
    {
    	super(key, msg, isFatal);
    	this.min = min;
    	this.max = max;
    }

    public AffirmStringLength(Object key, boolean isFatal, int min, int max)
    {
    	this(key, "Value must be a valid sequence of characters", isFatal, min, max);
    }

    public AffirmStringLength(Object key, int min, int max)
    {
    	this(key, "Value must be a valid sequence of characters", true, min, max);
    }

    public Object affirm(Object val) throws AffirmationException
    {
        // already ok?
        if (val instanceof String) {
	    int len = val.toString().length();
	    if (len >= min && len <= max)
		return val;
	    else
		throw new AffirmationException(msg, isFatal);
       	} else {
	    throw new AffirmationException(msg, isFatal);
	}
    }

    private int min;
    private int max;
}


