/**
 * $Id: AffirmationException.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package org.inxar.affirm;

/**
 *<code>AffirmationException</code> are thrown by <code>Affirmation</code> implementations if
 * and when the input they are affirming is discovered to be invalid.  An error message and a flag are carried through on the <code>AffirmationException</code>
 * back to the caller in the <code>catch</code> exception handling clause.  The <code>isFatal</code> flag
 * may be used to indicate that the error is severe enough such that further affirmations, if planned,
 * should not be executed.
 */
public class AffirmationException
extends Exception
{
	/**
	 * Contructs a new exception with an error message (eventually headed to a <code>Detail</code>
	 * in typical usage), the key, the value of (the bad data) itself, and the aforementioned flag.
	 */
	public AffirmationException(String msg, boolean isFatal)
	{
		super(msg);
		this.isFatal = isFatal;
	}

	/**
	 * Flag to indicate if further affirmations for this key should be aborted (if <code>true</code>).
	 */
	public final boolean isFatal;
}

