/**
 * $Id: Data.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package org.inxar.affirm;

/**
 * The <code>Data</code> interface is the 'goal' object in that it is
 * the carrier of information that has been successfully verified
 * through the proclamation/affirmation methodology.  It acts as a
 * map.  Any individial <code>Datum</code> may be accessed with the
 * <code>get()</code> method.  Additionally, <code>getXXX()</code>
 * methods are defined for all the primitive types which act as
 * shortcuts.  It is expected that the user of the <code>Data</code>
 * object (1) has set up affirmations appropriately and (2) knows
 * their data well enough to decide where it is appropriate to use the
 * shortcut methods.  This is important since the primitive return
 * value for an undefined key may mimic a real value since there is no
 * concept of <code>null</code> with primitive types.  
 */
public interface Data
{
    /**
     * Returns an enumeration over all the <code>Datum</code> objects
     * in the set.  
     */
    java.util.Enumeration enumerator();

    /**
     * Returns the <code>Datum</code> mapped to the given key.  
     */
    Datum get(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as a boolean.
     * @return the value under this key or <code>false</code> if no
     * such key exists.  
     */
    boolean getBoolean(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as a byte.
     * @return the value under this key or <code>0</code> if no such
     * key exists.  
     */
    byte getByte(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as a short.
     * @return the value under this key or <code>0</code> if no such
     * key exists.  
     */
    short getShort(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as an int.
     * @return the value under this key or <code>0</code> if no such
     * key exists.  
     */
    int getInt(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as a long.
     * @return the value under this key or <code>0</code> if no such
     * key exists.  
     */
    long getLong(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as a float.
     * @return the value under this key or <code>0.0</code> if no such
     * key exists.  
     */
    float getFloat(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as a double.
     * @return the value under this key or <code>0.0</code> if no such
     * key exists.  
     */
    double getDouble(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as a char.
     * @return the value under this key or <code>(char)0</code> if no
     * such key exists.  
     */
    char getChar(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as a string.
     * @return the value under this key or <code>null</code> if no
     * such key exists.  
     */
    String getString(Object key);

    /**
     * Returns the <code>Datum</code> mapped to the given key as an object.
     * @return the value under this key or <code>null</code> if no
     * such key exists.  
     */
    Object getObject(Object key);

    /**
     * Returns <code>true</code> if no mappings exist in this object,
     * <code>false</code> otherwise.  
     */
    boolean isEmpty();

    /**
     * Returns the number of key value mappings in the set.  
     */
    int size();
}

