/**
 * $Id: Detail.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package org.inxar.affirm;

/**
 * A <code>Detail</code> acts as a container for one particular error
 * message that originated from a single invalid
 * <code>Affirmation.affirm(Object)</code> invocation.  
 */
public interface Detail
{
    /**
     * Returns the argument value which caused the
     * <code>Affirmation.affirm(Input)</code> to fail.  
     */
    Object getValue();

    /**
     * Returns an error message that originated from the
     * <code>Affirmation</code> object.  
     */
    String getMessage();

    /**
     * <code>Detail</code> objects are typically implemented as a
     * linked list.  To run through all the error details for a
     * certain key, evaluate a <code>do .. while</code> statment until
     * the <code>Detail.hasNext()</code> method returns
     * <code>false</code>.  
     */
    boolean hasNext();

    /**
     * Returns the next detail in the list or null if this is the last
     * one.  
     */
    Detail next();
}

