/**
 * $Id: ProclamationException.java,v 1.1.1.1 2000/12/08 08:22:53 pcj Exp $
 *
 * Copyright (C) 2000 Paul Cody Johnston - paul@inxar.org
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.

 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
package org.inxar.affirm;

/**
 * <code>ProclamationException</code> instances are thrown by
 * <code>Proclamation</code> objects to indicate that at least one
 * error has occurred while verifying the input using applied
 * <code>Affirmation</code> implementations.  The exception acts
 * mainly as a carrier of two objects -- the <code>Data</code> and the
 * <code>Errata</code> containers, which hold the partitioned 'good'
 * and 'bad' data, respectively.  
 */
public class ProclamationException
    extends Exception
{
    /**
     * Builds a new <code>ProclamationException</code> with the given
     * <code>Data</code> and <code>Errata</code> objects.  If either
     * argument is null, the instance will switch over to an empty
     * implementation.  Thus, the references of the
     * <code>ProclamationException.data</code> and
     * <code>ProclamationException.errata</code> fields are guaranteed
     * to never be null, though the containers that live there may be
     * empty.  
     */
    public ProclamationException(Data data, Errata errata)
    {
	this.data = data == null ? DATA : data;
	this.errata = errata == null ? ERRATA : errata;
    }
    
    /**
     * Reference to the rescued <code>Data</code> object.  
     */
    public Data data;
    
    /**
     * Reference to the rescued <code>Errata</code> object.
     */
    public Errata errata;
    private static final Data DATA  = new EmptyData();
    private static final Errata ERRATA = new EmptyErrata();
    private static final java.util.Enumeration ENUM = new EmptyEnumerator();
    
    private static final class EmptyData
	implements Data
    {
    	public java.util.Enumeration enumerator() { return ENUM; }
    	public Datum get(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public boolean getBoolean(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public byte getByte(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public short getShort(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public int getInt(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public long getLong(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public float getFloat(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public double getDouble(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public char getChar(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public String getString(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public Object getObject(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public boolean isEmpty() { return true; }
	public int size() { return 0; }
    }
    
    private static final class EmptyErrata
	implements Errata
    {
    	public java.util.Enumeration enumerator() { return ENUM; }
    	public Erratum get(Object key) { throw new RuntimeException("This implementation does not support this operation"); }
	public boolean isEmpty() { return true; }
	public int size() { return 0; }
    }
    
    private static final class EmptyEnumerator
	implements java.util.Enumeration
    {
	public boolean hasMoreElements() { return false; }
	public Object nextElement() { throw new RuntimeException("This implementation does not support this operation"); }
    }
}
